@@ -25,6 +25,7 @@ from . import _jproxy
 from . import _jexception
 from . import _jcollection
 from . import _jobject
+from . import _jio
 from . import _properties
 from . import nio
 from . import reflect
@@ -36,6 +37,25 @@ def setUsePythonThreadForDeamon(v):
     global _usePythonThreadForDaemon
     _usePythonThreadForDaemon = v
 
+_initializers=[]
+
+def registerJVMInitializer(func):
+    """Register a function to be called after jvm is started"""
+    _initializers.append(func)
+
+def _initialize():
+    _properties._initialize()
+    _jclass._initialize()
+    _jarray._initialize()
+    _jwrapper._initialize()
+    _jproxy._initialize()
+    _jexception._initialize()
+    _jcollection._initialize()
+    _jobject._initialize()
+    nio._initialize()
+    reflect._initialize()
+    for func in _initializers:
+        func()
 
 def isJVMStarted() :
     return _jpype.isStarted()
@@ -48,16 +68,7 @@ def startJVM(jvm, *args):
     :param args: Arguments to give to the JVM
     """
     _jpype.startup(jvm, tuple(args), True)
-    _jclass._initialize()
-    _jarray._initialize()
-    _jwrapper._initialize()
-    _jproxy._initialize()
-    _jexception._initialize()
-    _jcollection._initialize()
-    _jobject._initialize()
-    _properties._initialize()
-    nio._initialize()
-    reflect._initialize()
+    _initialize()
 
     # start the reference daemon thread
     if _usePythonThreadForDaemon:
@@ -67,15 +78,7 @@ def startJVM(jvm, *args):
 
 def attachToJVM(jvm):
     _jpype.attach(jvm)
-
-    _jclass._initialize()
-    _jarray._initialize()
-    _jwrapper._initialize()
-    _jproxy._initialize()
-    _jexception._initialize()
-    _jcollection._initialize()
-    _jobject._initialize()
-    _properties._initialize()
+    _initialize()
 
 def shutdownJVM():
     _refdaemon.stop()
@@ -98,8 +101,12 @@ def get_default_jvm_path():
     :return: The path to the JVM shared library file
     :raise ValueError: No JVM library found
     """
-    if sys.platform in ("win32", "cygwin"):
-        # Windows or Cygwin
+    if sys.platform == "cygwin":
+        # Cygwin
+        from ._cygwin import WindowsJVMFinder
+        finder = WindowsJVMFinder()
+    elif sys.platform == "win32":
+        # Windows
         from ._windows import WindowsJVMFinder
         finder = WindowsJVMFinder()
     elif sys.platform == "darwin":
