@@ -2,13 +2,13 @@ from jpype import *
 import time
 
 def run():
-    print 'Thread started'
+    print('Thread started')
     try:
-        print repr(java.awt.Frame)
+        print(repr(java.awt.Frame))
         javax.swing.JFrame("Test Frame").setVisible(True)
         shutdownGuiEnvironment()
-    except JException, ex :
-        print ex
+    except JException as ex :
+        print(ex)
 
 
 startJVM(getDefaultJVMPath())
